

"""
BrahmaCore configuration file. Included:

1 - Standard python imports
2 - BrahmaLogging Formatter
3 - Standard output paths {e.g. log folder}

written by Rahul Tandon for Vixen Capital 2020. 
rahul@vixencapital.com
  
"""

#standard imports
import os, sys, random

'''
APP_FOLDER = os.path.join(os.path.expanduser('~'),'dev','app','tutorial-bot','')
sys.path.insert(0,APP_FOLDER)
from brahmaAppConf import *
'''
#
#path definitions {e.g. log folder(s)} All paths set to just add filename
#

#---brahma core root. IMPORTANT as other paths are ALL relative to this
#edit for dist. Don't want paths as package installed
#DO NOT TOUCH!!!
#BRAHMA_CORE_ROOT_PATH  = os.environ['BRAHMA_CORE_ROOT_PATH']
#BRAHMA_CORE_LOG_PATH = BRAHMA_CORE_ROOT_PATH + os.path.join('logs','')
#os.environ['BRAHMA_CORE_LOG_PATH'] = str(BRAHMA_CORE_LOG_PATH)

#BRAHMA_GENOME_CODE_ROOT_PATH =  BRAHMA_CORE_ROOT_PATH + os.path.join('code','genome','')
#BRAHMA_DNA_CODE_ROOT_PATH =  BRAHMA_CORE_ROOT_PATH + os.path.join('code','dna','')
#BRAHMA_GENETIC_DATA_FOLDER_LCL = BRAHMA_CORE_ROOT_PATH+ os.path.join('code','genes','') 
#BRAHMA_WORLD_PATH = BRAHMA_CORE_ROOT_PATH + os.path.join('code','world','')
#BRAHMA_BOT_ROOT_PATH = BRAHMA_CORE_ROOT_PATH + os.path.join('code','bots','')
#os.environ['BOT_SOURCE_LCL'] = BRAHMA_BOT_ROOT_PATH

#
#add brahma imports
#


#add dna code to system
#sys.path.insert(0,BRAHMA_DNA_CODE_ROOT_PATH)

#add genome code to system
#sys.path.insert(0,BRAHMA_GENOME_CODE_ROOT_PATH)

#add local genes repo
#sys.path.insert(0,BRAHMA_GENETIC_DATA_FOLDER_LCL)

#add environment 
#sys.path.insert(0,BRAHMA_WORLD_PATH)

#add bot root
#sys.path.insert(0,BRAHMA_BOT_ROOT_PATH)


#import brahma logging format
#from brahmaLogging import *
#logger = GetBrahmaLogger("CORE_CONFIG")

#import dna
#logger.debug("importing DNA from : {0}".format(BRAHMA_DNA_CODE_ROOT_PATH))
#from vixen_dna import *

#import genes -  only lcl brahma ones
'''
logger.debug("importing local genes from : {0}".format(BRAHMA_GENETIC_DATA_FOLDER_LCL))
'''
'''
from g_DownFractalPresent import *
from g_UpFractalPresent import *
from risk_RiskGeneAlpha import *
'''


#import genome
#logger.debug("importing Genome from : {0}".format(BRAHMA_GENOME_CODE_ROOT_PATH))
#from vixen_genome import *

#import environment -> static and dynamic
#logger.debug("importing World from : {0}".format(BRAHMA_WORLD_PATH))
#from population import *
#from dataSeries import *
#from simulatedfeed import *

#import bot root
#logger.debug("importing Bot Root from : {0}".format(BRAHMA_BOT_ROOT_PATH))
#from bot_root.py import *

#run flags



DEBUG = False
GENOME_DEBUG = False
ENV_DEBUG = False
SP_DEBUG = False
RUN_DEBUG = False
EXPRESS_DEBUG = False
TRADER_DEBUG = False
DNA_EXPRESS_DEBUG = False
TRANSCRIPTION_DEBUG = False
DECISION_DEBUG = False
DECISION_SUMMARY_DEBUG = True
DECISION_STATE_DEBUG = False
DECISION_ADDING_MEC = False
FITNESS_DEBUG = False
EVOLUTION_ALLOWED = True
OPEN_TRADE = False
LIVE_TRANSCRIPTION_DATA = False
#data output
POOL_TRADER_DATA = False
TOURNAMENT_DEBUG = False
CHILD_DEBUG = False
STRUCTURE_OUTPUT = False
MOOD_OUTPUT = False
PL_OUTPUT = False
PL_OUTPUT_LOCAL = True
#CandleBuckets = [1, 2, 4, 5, 10, 15, 20, 30]
CandleBuckets = [1, 2]
generationWinners = True

MAX_DECISIONS = 100
OUTPUT_ITERATION_DATA = True

#logger.info("configCore file read.")
#EpochBuckets = [1, 2, 4, 5, 10, 15, 20, 30]
EpochBuckets = [1, 2]

'''
from dnaroot import *
from g_DownFractalPresent import *
from g_UpFractalPresent import *
from risk_RiskGeneAlpha import *
'''



import os, sys

#config core
#sys.path.append('../') #core code root ( /brahama/code/ )
from brahma.configCore import *

#!!!!!!!!!!!!!!!!!!!!
#must edit this
#from g_DownFractalPresent import *
#from g_UpFractalPresent import *
#from risk_RiskGeneAlpha import *
#from g_DeltaValue import *


class VixenGenome(object):

    """
    Genome for Vixen's DNA. The genome, located within the DNA structure, 
    defines the genes within the individual. Each gene interacts with the data vector
    of discretised environmental conditions and will ultimately adopt an on/off state.
    These differing states will provide an excitement level/expression for the genome
    in which they sit. The expression level (e) of the genome will satisfy {0 < e < 1}
    
    IMPORTANT: Genomes within the same DNA structure will all recieve the same 
    size vector of data and resident genes must be able to adopt a state with any 
    given data vector sizes. 

    This 'VixenGenome' is considered the root of all genome structures. Genomes must be
    able to,  at the very minimum, build & express the genome.
    
    Arguments:
        object {[type]} -- [description]

    Returns:
        [type] -- [description]
    """
   

    def __init__(self, minSize=1, maxSize=1, envName="test", genomeType=""):
        """
        Creates a genome structure

        Args:
            minSize (int, optional): min number of genes. Defaults to 1.
            maxSize (int, optional): max number of genes. Defaults to 1.
            envName (str, optional): name of environment. Defaults to "test".
            genomeType (str, optional): [description]. Defaults to "".
        """
        import random

        self.expressionLevel = 0    #expression level of the genome
        self.genomeSize=random.randint(int(minSize), int(maxSize)) #number of genes in the genome
        self.envName = envName #environment within which the genome interacts {e.g. EURUSD}
        self.genomeTag = "genome" + str(random.randint(100, 10000)) #unique id of the genome
        self.genome = {} #hashtag of all genes in the genome
        self.genomeExpression = {} #expression of gene tagged by geneID
        self.geneticSelectionDict = {} #dict of gene types from which to select TODO: apply number contraints
        #nov 2020
        #self.riskGeneticSelectionDict = {} #dict of risk genes from which to select 
        self.genomeType = genomeType  #type of genome { normal, risk }  NOTE: will depracate soon  
        

        #self.logger = GetBrahmaLogger("GENOME")


    def __str__(self):
        """
        Override print method for description of Genome and all genes ( or conditions )

        Returns:
            string: Description of genome (can be used in logging / debugging)
        """
        returnStr = ""
        returnStr += ("Tag: {2} Size: {0} ; Environment {1}".format(self.genomeSize, self.envName, self.genomeTag)) + "\n"
        returnStr += "*************** DETAILS ************ \n"

        for geneTag, gene in self.genome.items():
            returnStr += ("Tag: {0} Type: {1}".format(gene.i_D, gene.condition)) + "\n"

        return returnStr

    def __exit__(self):
        pass
        #del self.logger
    
    def BuildGenome(self):
        
        """
        Build a genome >
        >. determine type of genome
        >. build a selection vector of available genes
        >. build and attach the genes to the genome.
        """
        

        if self.genomeType=="risk":
            #self.logger.info("Building Risk Genome - Gene count: {0}".format(self.genomeSize))
            #build list of avilable risk genes
            self.BuildRiskGeneSelectionDict()

            for geneCnt in range(0, self.genomeSize):


                
                geneType = self.SelectRiskGeneFromSelectionDict()
                #logger.info("Risk gene selected: {0}".format(geneType))
                geneStructureTmp = self.BuildGeneStructure(geneType, self.riskGeneticSelectionDict)
                #self.genome[geneStructureTmp.i_D] = geneStructureTmp
                #self.logger.info("Gene built: {0}".format(geneType))
            #attach to genome
                #kill tmp structure
                geneStructureTmp = None




            return

        #self.logger.info("Building Normal Genome - Gene count: {0}".format(self.genomeSize))
        self.BuildGeneSelectionDict()

        for geneCnt in range(0, self.genomeSize):
            #select
            geneType = self.SelectGeneFromSelectionDict()
            #self.logger.debug("Normal gene selected: {0}".format(geneType))

            #build & get tagID
            geneStructureTmp = self.BuildGeneStructure(geneType,  self.geneticSelectionDict)
            #self.logger.info("Gene built: {0}".format(geneType))
            #attach to genome

            self.genome[geneStructureTmp.i_D] = geneStructureTmp
            #kill tmp structure
            geneStructureTmp = None




    def BuildRiskGeneSelectionDict(self):

        import os
        geneFileList = os.listdir(BRAHMA_GENETIC_DATA_FOLDER_LCL)
        for fileName in geneFileList:
            if fileName.endswith('.py'):
                fnArray = fileName.split('.')
                geneType = fnArray[0]
                if geneType != "dnaroot":
                    prefix = geneType.split('_')
                    prefixStr = prefix[0]
                    if prefixStr == "risk":
                        geneName = prefix[1]
                        #logger.info("Adding Gene to Selection: {0}".format(geneName))
                        self.riskGeneticSelectionDict[geneName] = eval(geneName)

    def SelectRiskGeneFromSelectionDict(self):
        import random
        
        try:
            #self.logger.debug("Trying to grab a random risk gene tag.")
            randomGeneTag = random.choice(list(self.riskGeneticSelectionDict))
            #self.logger.debug("Random gene tag: {0}".format(randomGeneTag))
            
            return randomGeneTag

        except Exception as e:
            #self.logger.critical("type error: " + str(e))
            #self.logger.critical(traceback.format_exc())
            #self.logger.critical("Error selecting risk gene")
            exit(0)
        return -1


    def BuildGeneSelectionDict(self):
        """Build gene type selection lists
        """


        import os
        #local genes loaded {todo: custom genes}
        #geneFileList = os.listdir(BRAHMA_GENETIC_DATA_FOLDER_LCL)
        #edit
        genetic_material_folder = USER = os.getenv('GENETIC_DATA_FOLDER_USR')
        geneFileList = os.listdir(genetic_material_folder)
        for fileName in geneFileList:
            
            if fileName.endswith('.py'):
                fnArray = fileName.split('.')
                geneType = fnArray[0]
                if geneType != "dnaroot":
                    prefix = geneType.split('_')
                    prefixStr = prefix[0]
                    if prefixStr == "g":
                        geneName = prefix[1]
                        #logger.info("Adding Gene to Selection: {0}".format(geneName))
                        self.geneticSelectionDict[geneName] = eval(geneName)

    def SelectGeneFromSelectionDict(self):
        """Select a genetype from the list
        and return to caller
        """
        import random
        try:
            #self.logger.debug("Trying to grab a random gene tag.")
            
            randomGeneTag = random.choice(list(self.geneticSelectionDict))
            #self.logger.debug("Random gene tag: {0}".format(randomGeneTag))
            
            return randomGeneTag

        except Exception as e:
            
            #self.logger.critical("Error selecting gene")
            exit(-1)
        return -1


    def BuildGeneStructure(self, gene_type = "", classCollection=None):
        if gene_type != "":
            pass
        else:
            #self.logger.critical("No genetic material found for gene_type: {0}".format(gene_type))
            exit("No genetic material")

        
        tmpGene = None
        #self.logger.debug("attempting to create gene from new selection approach: {0}".format(gene_type))
        try:
            #print (gene_type)
            #print (self.geneticSelectionDict)
            #tmpGene = self.geneticSelectionDict[gene_type]("test")
            tmpGene = classCollection[gene_type]("test")
        except Exception as e:
            #self.logger.critical("Attempting to create gene from new selection approach: FAIL")
            
            exit(0)

        return tmpGene

        ''' lets get rid of this...
        if gene_type == "direction_negative_alpha":
            tmpGene = DirectionNegativeAlpha(mkt=self.envName)

        if gene_type == "direction_positive_alpha":
            tmpGene = DirectionPositiveAlpha(mkt=self.envName)

        if gene_type == "down_fractal_present":
            tmpGene = DownFractalPresent(mkt=self.envName)

        if gene_type == "up_fractal_present":
            tmpGene = UpFractalPresent(mkt=self.Market)

        if gene_type == "risk_alpha":
            tmpGene =  RiskGeneAlpha(mkt=self.Market)

        if gene_type == "risk_beta":
            tmpGene =  RiskGeneBeta(mkt=self.Market)
        '''



       

    def express(self ,data):
        """[summary]

        :param data: [description]
        :type data: [type]
        :return: [description]
        :rtype: [type]
        """
        expressSum = 0
        
        #debug  -> variable expression
        #print ("genome size:{0}".format(len(self.genome)))

        for geneTag, gene in self.genome.items():
            if EXPRESS_DEBUG:
                pass
                #self.logger.debug("Genome expressing")

            #get express value
            expression = 0
            expression = gene.run(data)
            #---debug - structure value analysis. 
            #print ("gene e: {0}".format(expression))
            #if expression == 0:
            #    print ("--------------")


            #-----
            self.genomeExpression[geneTag] = expression
            if GENOME_DEBUG:
                if gene.SafeRun == False:
                    pass
                    #self.logger.debug("Gene error. Not a safe run. {0}".format(gene.condition))
            expressSum = expressSum + expression
            #debug  -> variable expression
            #print("expressSum {0}".format(expressSum))
            #print("gene: {0}".format(expression))
            #print("gene_c: {0}".format(gene.condition))


            if EXPRESS_DEBUG:
                pass
                #self.logger.debug("Gene Expression : {0}".format(expression))



        if EXPRESS_DEBUG:
            pass
            #self.logger.debug("Genome Expression Vector:")
            #self.logger.debug(self.print_genome_expression())


        #edit expression -> expressSunm Nov 2020
        if expressSum != 0:
            self.ExpressionLevel = float(expressSum/self.genomeSize)

            #print("final expressSum {0}".format(expressSum))
        else:
            self.ExpressionLevel = 0

        #debug - volatile
    

        #debug - volatile
        #print ("genome expression  value: {0}".format(self.ExpressionLevel))


         #debug ---> gene flip analysis
            
        return self.ExpressionLevel




    def print_genome_expression(self):
        for geneTag, expressionVal in self.GenomeExpression.items():
            print ("Gene Tag: {0} ; GeneExpression {1}".format(geneTag, expressionVal))
    def get_structure(self):
        structure = {}
        genome_structure = {}
        genome_structure['numberGenes'] = self.GenomeSize
        gene_structures = {}
        for geneTag, gene in self.Genome.items():
            pt_structures = {}
            pt_structures['condition'] = gene.Condition
            pt_structures['id'] = gene.I_D
            pt_structures['dob'] = gene.DOB
            gene_structures[geneTag] = pt_structures

        structure['genome_meta'] = genome_structure
        structure['genes'] = gene_structures

        return structure





import os, sys, time

CONFIGPATH = os.path.join(os.path.expanduser('~'), 'tradingalgorithm', 'code', 'lucen', 'code', '')
#---add config folder to project
sys.path.insert(0,CONFIGPATH)


#---import custom config
from config import *

#---required libs
from lib import *


class Candle(object):

    def __init__(self, priceType="BID"):
        self.high = 0
        self.low = 10000000
        self.open = 0
        self.close = 0
        self.ticks = 0
        self.time = 0
        self.priceType = priceType
        self.minuteCounter = -1
        self.complete = False
        self.direction = 0


    def AddPriceTick(self, bid, offer, time):

        if bid>self.high:
            self.high = bid

        if bid<self.low:
            self.low = bid

        if self.ticks is 0:
            self.open = bid
            #print ("OpEN BID: %f" % (bid))

        self.close = bid
        self.ticks += 1
        self.time = time

        if self.open > self.close:
            self.direction = -1

        if self.open < self.close:
            self.direction = 1



    def __str__(self):
        return ("O: %f C: %f H: %f L: %f M:%s C: %s" % (self.open, self.close, self.high, self.low, self.minuteCounter, self.complete))





class CandleHolder(object):

    def __init__(self, deltaT = 1): #made into integer
        self.deltaT = deltaT
        self.candles = None
        self.fresh = True

        self.deltaTFloat = float(deltaT)
        

        #if deltaT is '1M':
        #    self.deltaTFloat = 1.0
        #if deltaT is '5M':
        #    self.deltaTFloat = 5.0

    def __str__(self):
        return ("%s : %f " % (self.deltaT, self.deltaTFloat))


    def Output(self):
        if self.candles is not None:
            for candle in self.candles:
                print(candle)

    def AddPrice(self, bid, offer, time):
        #print ("[cs] bid: %f offer: %f time: %s " % (bid, offer, time))
        #make sure we have a candle
        if self.candles is None:
            self.candles = []
            candle = Candle(priceType= "BID")
            self.candles.append(candle)

        minute = GetMinute(time)
        ActiveCandle = self.candles[len(self.candles)-1]
        if self.fresh is True:
            self.candles[0].minuteCounter = minute




        #add price to candle

        #add to existing candle

        #sort out new candle or not depending on delta T*************



        if self.deltaT == 1:
            if ActiveCandle.minuteCounter == minute:
                #add to candle
                ActiveCandle.AddPriceTick(bid, offer, time)
                self.fresh = False
                return 0

            else:
                #new candle
                #print completed cs
                #print (ActiveCandle)
                ActiveCandle.complete = True
                #print (self.candles[len(self.candles)-1])
                candle = Candle(priceType= "BID")
                candle.minuteCounter = minute
                self.candles.append(candle)
                ActiveCandle = self.candles[len(self.candles)-1]
                ActiveCandle.AddPriceTick(bid, offer, time)
                self.fresh = False
                return 0

        newCandle = False
        if self.deltaT != 1:
            mod_val = float(minute) % float(self.deltaTFloat)
            if int(mod_val) is 0:
                newCandle = True
                #print ("FSA")
                #exit()
                #print (("%f %f %f") % (mod_val, float(minute), float(self.deltaTFloat)))
        '''
        if ActiveCandle.minuteCounter == minute:
            #add to candle
            ActiveCandle.AddPriceTick(bid, offer, time)

        else:
            #new candle
            #print completed cs
            #print (ActiveCandle)
            ActiveCandle.complete = True
            #print (self.candles[len(self.candles)-1])
            candle = Candle(priceType= "BID")
            candle.minuteCounter = minute
            self.candles.append(candle)
            ActiveCandle = self.candles[len(self.candles)-1]
            ActiveCandle.AddPriceTick(bid, offer, time)
        '''



        if newCandle is True:
            if int(ActiveCandle.minuteCounter) != int(minute):
                #print ("%d %d" % (int(ActiveCandle.minuteCounter), int(minute)))
                #new candle
                #print completed cs
                #print (ActiveCandle)
                ActiveCandle.complete = True
                #print (self.candles[len(self.candles)-1])
                candle = Candle(priceType= "BID")
                candle.minuteCounter = minute
                self.candles.append(candle)
                ActiveCandle = self.candles[len(self.candles)-1]
                ActiveCandle.AddPriceTick(bid, offer, time)
                self.fresh = False
                #print ("add to new")

        if newCandle is False:
            ActiveCandle.AddPriceTick(bid, offer, time)
            self.fresh = False
            #print ("add to exist: %s" % minute)



"""Transcription genetic material/process is responsible for building the protein.
Here we model the action of decision making as the bot able to transcrible protein.
All the dna material in the bot is the genetic material required to regulate and transcribe
three different protein types:
1. An enzyme whose presence prescribes a 'ACTIVE' state
2. An enzyme whose presence prescribes a 'WAIT' state

This class/module is also considered 'genetic material', but it; sole responsibilty is to transcribe the already
exisiting genetic material. The genetic state of the gene(s) modelled here is/are always on (1/True).

gene state->genome->chromosome->expression vector-> [environmental regulation] -> transcription

[environmental regulation] will simply promote/suppress the expression vector rather than work on the transcrition
state

"""
import random, logging


class Transcription(object):
    """This class models the genetic material required to transcribe the 'bot's' genetic material.
    It reads in chromosomal expression levels and decides whether

    Arguments:
        object {[type]} -- [description]


    """

    def __init__(self):

        self.transcription_state = None
        self.transcription_state_recorder = {}
        #self.transcription_threshold = random.random()
        #self.transcription_threshold = 0.002
        self.transcription_threshold = random.random()
        #v2. update - Apr 16
        self.transcription_threshold_exit = random.random()

    def __str__(self):
        return ("Activation Level: {0}".format(self.transcription_threshold))

    def get_structure(self):
        transcription_str = {}
        transcription_str['state'] = self.transcription_state
        transcription_str['threshold'] = self.transcription_threshold_exit

    def transcribe(self, expression_data):
        """Transcribe DNA into protein

        Arguments:
            expression_data {[type]} -- [description]

        Returns:
            [type] -- [description]
        """
        #print ("state {0}".format(self.transcription_state))
        #check risk transcription first

        #risk
        risk_expression_str = expression_data['risk_expression_data']

        risk_expression_vec = []

        for chromTag, expression in risk_expression_str.items():
            risk_expression_vec.append(expression)

            #print (expression)
            if expression == 1:
                protein_transcribe = False
                self.transcription_state = False
                #print ("Exiting: {0} {1}".format(expression_data['pl'], expression_data['name']))
                logger.critical("Exiting from risk: {0} {1}".format(expression_data['pl'], expression_data['name']))
                return protein_transcribe



        #not risk
        expression_str = expression_data['expression_data']

        expression_vec = []

        for chromTag, expression in expression_str.items():
            expression_vec.append(expression)

        if TRANSCRIPTION_DEBUG:
            print ("Expression vector sent: ")
            print (expression_vec)

        protein_transcribe = self.map_expression_vector(expression_vec)
        #protein_transcribe = True

        if protein_transcribe == True:

            if TRANSCRIPTION_DEBUG:
                print ("Transcribing")
            self.transcription_state = True
            return protein_transcribe
        '''
        if protein_transcribe == False:

            if self.transcription_state == True:
                logger.critical("Exiting from threshold, not risk: {0}".format(expression_data['pl']))
            if TRANSCRIPTION_DEBUG:
                print ("Not Transcribing")
            self.transcription_state = False
            #print (expression_data['pl'])

            return protein_transcribe
        '''

    def map_expression_vector(self, e_data=None):
        """Map expression vector. State function on expression vector to determine
        transcription state.

        Keyword Arguments:
            e_vector {[type]} -- [description] (default: {None})
        """
        express_sum = 0
        number_express = len(e_data)

        for expression in e_data:
            express_sum = express_sum + expression

        transcription_activity = express_sum/number_express
        #see if trade triggered
        if transcription_activity > self.transcription_threshold:

            if TRANSCRIPTION_DEBUG:
                print ("activity: {0} threshold: {1}".format(transcription_activity, self.transcription_threshold))
            return True


        '''
        #if not already in trade
        if self.transcription_state == False:

            if transcription_activity > self.transcription_threshold:

                if TRANSCRIPTION_DEBUG:
                    print ("activity: {0} threshold: {1}".format(transcription_activity, self.transcription_threshold))
                return True


            else:

                return False
        '''
        '''
        if self.transcription_state == True:
            #stay in trade
            print (":SFSFS")
            return True
        '''



        '''
        #if already in trade
        if self.transcription_state == True:

            if transcription_activity < self.transcription_threshold_exit:
                if TRANSCRIPTION_DEBUG:
                    print ("activity: {0} threshold: {1}".format(transcription_activity, self.transcription_threshold_exit))
                #return False






        else:
            if TRANSCRIPTION_DEBUG:
                print ("activity: {0} threshold: {1}".format(transcription_activity, self.transcription_threshold))
            #return False

        '''



    def mutate(self):
        self.transcription_threshold = random.random()
        self.transcription_threshold_exit = random.random()


if __name__=="__main__":
    trans = Transcription()
    data = { 'expression_data':[0.3,0,0.65,0] }
    trans.transcribe(data)



#from configCore import *
import logging





def GetBrahmaLogger(source='undef'):
    """[summary]

    Args:
        source (str, optional): Source module/class of log message. Defaults to 'undef'.

    Returns:
        logger: logger handle
    """
    
    # define BRAHMA_CORE_LOG_PATH
    import os
    
    BRAHMA_CORE_LOG_PATH = os.environ['BRAHMA_APP_LOG_PATH']

    # create logger with 'spam_application'
    logger = logging.getLogger(source)

    # create console handler with a higher log level
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)

    # create file handler with a higher log level
    fh = logging.FileHandler(BRAHMA_CORE_LOG_PATH + "brahmalog.txt", 'a')
    fh.setLevel(logging.DEBUG)

    #format logging
    ch.setFormatter(BrahmaLogFormatter())
    fh.setFormatter(BrahmaLogFormatter())

    #set logging level
    logger.setLevel(logging.DEBUG)

    #attach streams to logger
    logger.addHandler(ch)
    logger.addHandler(fh)

    return logger


class BrahmaLogFormatter(logging.Formatter):
  """
  BrahamLogger provides the formatting required for all logging
  in brahmaCore. 

  Args:
      Level (String): Level of logging output {debug, error, ...}
  """
  grey = "\x1b[38;21m"
  yellow = "\x1b[33;21m"
  red = "\x1b[31;21m"
  bold_red = "\x1b[31;1m"
  reset = "\x1b[0m"
  format = "%(asctime)s - %(name)-20s - %(levelname)-10s - %(message)-40s (%(filename)s:%(lineno)d)"

  FORMATS = {
      logging.DEBUG: grey + format + reset,
      logging.INFO: grey + format + reset,
      logging.WARNING: yellow + format + reset,
      logging.ERROR: red + format + reset,
      logging.CRITICAL: bold_red + format + reset
  }

  def format(self, record):
      log_fmt = self.FORMATS.get(record.levelno)
      formatter = logging.Formatter(log_fmt)
      return formatter.format(record)

if __name__ == "__main__":
        
    # create logger with 'spam_application'
    logger = logging.getLogger("LoggingFormatter")

    # create console handler with a higher log level
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)

    # create file handler with a higher log level
    fh = logging.FileHandler(BRAHMA_CORE_LOG_PATH + "log.txt", 'w+')
    fh.setLevel(logging.DEBUG)



    if len(sys.argv)> 1:
        if sys.argv[1] == 'log':
            ch.setFormatter(logging.Formatter('%(asctime)s : %(levelname)s : %(name)s : %(message)s'))
            fh.setFormatter(logging.Formatter('%(asctime)s : %(levelname)s : %(name)s : %(message)s'))
            
        elif sys.argv[1] == 'color':
            ch.setFormatter(BrahmaLogFormatter())
            fh.setFormatter(BrahmaLogFormatter())

    if len(sys.argv) > 2:
        logger.setLevel(logging.__getattribute__(sys.argv[2]))
    else:
        logger.setLevel(logging.DEBUG)

    logger.addHandler(ch)
    logger.addHandler(fh)


    import random
    import time
    for _ in range(100):
        i = random.randint(0, 10)
        if i <= 4:
            logger.info("Value is {} - Everything is fine".format(i))
        elif i <= 6:
            logger.warning("Value is {} - System is getting hot".format(i))
        elif i <= 8:
            logger.error("Value is {} - Dangerous region".format(i))
        else:
            logger.critical("Maximum value reached")
        time.sleep(0.3)



import os, sys

#command line input

def getopts(argv):
    opts = {}  # Empty dictionary to store key-value pairs.
    while argv:  # While there are arguments left to parse...
        if argv[0][0] == '-':  # Found a "-name value" pair.
            opts[argv[0]] = argv[1]  # Add key and value to the dictionary.
        argv = argv[1:]  # Reduce the argument list by copying it starting from index 1.
    return opts

#raw data file manipulation

def parsePriceDataLine(line):
    parsedArray = []
    parsedArray = line.split(',')
    #print (parsedArray[1])
    return (parsedArray)

def GetDateStr(dt):
    year = str(dt.year)
    month = str(dt.month)
    if int(month) < 10:
        month = '0' + str(month)
    day = str(dt.day)
    if int(day) < 10:
        day = '0' + str(day)
    return (year+month+day)

def GetDayTime(dt):
    print (dt)
    hour = str(dt.hour)
    if int(hour) < 10:
        hour = '0' + str(hour)
    minute = str(dt.minute)
    if int(minute) < 10:
        minute = '0' + str(minute)
    return (hour+minute)

def parseHighLow(data):
    print ("waiting...")

def testlib():
    print ('connected to lib module')

def GetConvertVal(market):
    if market in marketConvertVals:
        return marketConvertVals[market]

def GetPipVal(market):
    if market in marketPipVals:
        return marketPipVals[market]

def GetTimeVecIndex(time, timeVec):
    return timeVec.index(time)

def GetPipDifference(startPrice, endPrice, market):
    deltaPrice = float(startPrice) - float(endPrice)
    pipVal = GetPipVal(market)
    deltaPips = deltaPrice/pipVal
    return round(deltaPips,2)

def GetPipValFromMarketPrice(deltaPrice, market):
    pipVal = GetPipVal(market)
    deltaPips = deltaPrice/pipVal
    return round(deltaPips,2)

def GetMinute(time):
    parsedArray = []
    parsedArray = time.split('-')
    return (parsedArray[len(parsedArray)-1])

def GetDateFromPricePrint(pricePrintTime):
    parsedArray = []

    time = str(pricePrintTime)
    parsedArray = time.split('-')
    #print (parsedArray)
    #print (parsedArray)
    #print(time)
    #print (parsedArray)
    date = parsedArray[0]+parsedArray[1]+parsedArray[2]
    return date

def getDate():
    import datetime
    now = datetime.datetime.now()

    #print (str(now))
    date = now.strftime("%Y-%m-%d")
    return date

def GetHourFromPricePrint(pricePrintTime):
    parsedArray = []

    time = str(pricePrintTime)
    parsedArray = time.split('-')
    #print (parsedArray)
    #print(time)
    
    hour = float(parsedArray[3])
    return hour
    #exit()

def GetMinuteFromPricePrint(pricePrintTime):
    parsedArray = []

    time = str(pricePrintTime)
    parsedArray = time.split('-')
    #print (parsedArray)
    #print(time)
    #print (parsedArray)
    minute = float(parsedArray[4])
    return minute
    #exit()

def GetPriceDeduction(price_one,  pips,market):
    pipVal = GetPipVal(market)
    diff = pips * pipVal
    return round(price_one - diff,5)


def GetPriceSum(price_one, pips, market):
    pipVal = GetPipVal(market)
    diff = pips * pipVal
    return round(price_one + diff, 5)


def GenerateTradeID():
    import random
    _id = str(random.randint(0,10000)) + str(random.randint(0,100000)) + "trade"
    return _id


#---------------------FTP DATA TO VX.COM SERVER---------------------------------------


def FTPFile(file='none'):
    from ftplib import FTP

    ftp = FTP('ftp.vixencapital.com')
    ftp.login(user='vixencapital.com', passwd='Edinburgh69!')

    UPLOADDIR = '/htdocs/plotter/data/'
    #filename = '../../results/dirbots/botsData.json'
    filename = "test.txt"
    #file = open(filename, 'rb')
    #ftp.cwd(UPLOADDIR)
    #ftp.storbinary('STOR'+filename, file)

    with open(filename, 'rb') as contents:
        ftp.storbinary('STOR %s' % filename, contents)

    ftp.quit()
    ftp.close()



"""
This module defines the environments within the brahma virtual world.
There are two main envirnents present:

1. The static environment: The static envirnment represents the population of 
bots/indis in the virtual world. {e.g. trading bots}

2. The dynamic envirnment: The dynamic environment provides the 'pressure' within
which the bots/indis live. {e.g. market price}

Coupled, these two environemnts interact to provide the source and pressure required
to evolve bots to live in the virtual environment.
"""

import os, sys
#insert root dir into system path for python
#config core
sys.path.append('../')
from configCore import *

class StaticEnvironment(object):
    """
    The Environment class provides the population for the brahmaCore game. 
    For example, in trading applicaiton, this would represent the popualtion
    of trading bots in the brahma virtual world.
    
    Arguments:
        object -- Root object
    """
    
    def __init__(self, op_parms=None):
        """
        initialise the Environment object. Optimisation paramters are required
        and simply represent the parameters for the game/optimsation.

        The logger for the module is also set.

        :param op_parms: These are the parameters for the 'game', defaults to None, but are required
        :type op_parms:key-value dictionary of game options, optional
        """
        self.optimisationParameters = op_parms

        #logger
        self.logger = GetBrahmaLogger("Environment")

        if DEBUG:
            self.print_parms()

    def PrintParms(self):
        """
        Simply display the parameters read by the game/optimisation.
        Output to log.
        """
        for key, value in self.OptimisationParameters.items():
            #print (key + " : " + str(value))
            self.logger.debug((key + " : " + str(value)))

    def BuildEnvironment(self):
        """
        This is the workhorse of the Environment class. Here we build the population
        that will be used in the game. User defined indivis/bots will have to be 
        used here via the Population class.
        """

        #---build population
        self.population = Population(self.optimisationParameters, name=self.optimisationParameters['population_name'])
        self.population.populate(species=self.optimisationParameters['species'])
        
        if DEBUG:
            self.population.show()

        #---deprecated
        #---build dynami nature (this will be what is used to apply pressure & select the best decision makers in the population)
        #self.NaturalWorld = nature



class DynamicEnvironment(object):
    """
    The dynamic aspect of the environment.Here we provide the force that will 
    apply pressure to the population and allow for a fitness distribution 
    required for evolution of bots/indis in the game.

    Arguments:
        object {class]} -- root class
    """

    def __init__(self):
        """
        Initialise the DynamicEnvironment class. Sets pressure conduit and pressure to none. 
        These key environmental variables will be initialised when the dynamic
        environment is built.
        """

        self.pressureConduit = None
        self.pressure = None

    def BuildEnvironment(self, parms=None, live=False):
        """
        Build the dynamic environment. Optimisation parameters will be required.

        :param parms: Optimisation parameters. This is initially defined by
        brahmaCore, but user defined variables can also be added for full 
        customisation.
        :type parms: Key value structure
        :param live: is the dynamic environment for use in an optimisation 
        capacity or forward running mode, defaults to False
        :type live: bool, optional
        :return: Status of  the pressureConduit and pressure components. [1,-1]
        :rtype: int
        """

        self.pressureSource = DataSeries()
        self.pressureSource.AddEnv(parms['env'])
        self.pressureSource.BuildDataStructures()

        if ENV_DEBUG and self.PressureConduit:
            print ("Pressure conduit setup")

        self.pressure = SimulatedFeed(market = parms['env'], numberOfDays = parms['numberEpochs'])
        self.pressure.BuildSimulatedFeed(parms['startData'], parms['endData'])
        self.pressure.SetDataSeries(self.pressureConduit)

        if ENV_DEBUG and self.Pressure:
            print ("Pressure source setup")

        if self.pressureConduit and self.pressure:
            print ("success")
            return 1

        else:
            return -1

if __name__ == "__main__":
    pass

    '''
    static_env = Environment(op_parms = OptimsationParameters)
    static_env.build_environment()
    dyn_env = DynamicEnvironment()
    dyn_env.build(OptimsationParameters)
    ''' 



"""Population class for evolutionary algorithms
"""



import os, sys, random
#insert root dir into system path for python
#config core
sys.path.append('../')
from configCore import *

#import root bot
from bot_root import *


class Population(object):
    """
    Population is the container object for all bots/indis in the game/optimisation run.
    
    Arguments:
        object {root object} -- object class
    """

    def __init__(self, parms=None, name="vx_dm"):
        """
        Create the Population class. This class holds all bots in the game.

        :param parms: optimisation parameters - from config file but passed through for now, defaults to None
        :type parms: key/value pairs, optional
        :param name: name root, defaults to "vx_dm"
        :type name: str, optional
        """
        #print ("fsdfw")
        self.name = name + str(random.randint(10,10000))
        self.parms = parms
        #bot names in list
        self.bots = []
        #species structure tagged by name
        self.species = {}
        #logger

        self.logger = GetBrahmaLogger("Population")

    def __exit__(self):
        del self.logger

    def Populate(self, species = "BotRoot", args = None):

        if species != "tribal":
            
            
            if DEBUG:
                print ("Size: " + str(self.parms['PopulationSize']))
            for i in range(0, int(self.parms['PopulationSize'])):
                if DEBUG:
                    print ("creating bot")
                self.CreateBot(species = species, args = args)
                
        else:

            self.CreateBot(species = species, args = args)

        # self.logger(self.show())
        self.show()

    def test(self):
        pass
    def CreateBot(self, species = None, args = None):
        self.botStr = {}
        self.botStr[species] = eval(species)
        

        #build the bot here - general
        bot_tmp = self.botStr[species](self.parms["env"], myspecies = species, myargs = args)
        bot_tmp.BuildBot(parms=self.parms)
        
        
        self.bots.append(bot_tmp.name)
        self.species[bot_tmp.name] = bot_tmp

        '''
        print ("Building.. " + species)
        bot_tmp = trader_template(self.Parms['market'])
        bot_tmp.build_trader(parms=self.Parms)
        self.Bots.append(bot_tmp.Name)
        self.Species[bot_tmp.Name] = bot_tmp
        bot_tmp = None
        '''

    def show(self):
        for key, value in self.species.items():
            #print (value)
            pass



if __name__ == "__main__":
    pass
    '''
    pop = Population(parms={'PopulationSize' : '100'}, name = "demo")
    pop.populate('trader')
    '''




import os, sys, random
CONFIGPATH = os.path.join(os.path.expanduser('~'), 'tradingalgorithm', 'code', 'lucen', 'code','')
sys.path.insert(0,CONFIGPATH)

from time import *
from config import *
from lib import *



#user defined [Lucen]
class PriceFeedPrint(object):
    """Price print from the simulated price feed. This is what is returned into the simulated feed 
    in the 'tick' function. This data is added to the connected market dataseries required for simulation
    or optimisation.
    
    Arguments:
        object {[type]} -- [description]
    
    Returns:
        [type] -- [description]
    """
    def __init__(self, bid, offer, time):
        """initialise the price print.
        
        Arguments:
            bid {[float/str]} -- the price print bid value
            offer {[float/str]} -- the price print off value
            time {[float/str]} -- the time of the price print
        """
        self.bid = float(bid)
        self.offer = float(offer)
        self.time = str(time)
        self.high = 0
        self.low = 0
        self.change = 0

    def __str__(self):
        """Returns the price print as a string for output.
        
        Returns:
            [string] -- [price print description]
        """
        return ("%f %f %s" % (self.bid, self.offer, self.time))

class SimulatedEpochFeed(object):
    """SimulatedDailyFeed stores the daily feed data for the market. Current resolution is one second.
    
    Arguments:
        object {[type]} -- [description]
    
    Returns:
        [type] -- [description]
    """
    def __init__(self, day):
        """Constructor. Set the day YY-MM-DD, bid offer and time vectors.
        
        Arguments:
            day {[type]} -- [description]
        """
        self.Day = day
        self.BidList = []
        self.OfferList = []
        self.TimeList =[]

    def AddPrint(self, bid, offer, time):
        """Add a print to the daily feed. This is called in the datastructure builder in SimulatedFeed.
        
        Arguments:
            bid {float} -- [bid]
            offer {float} -- [offer]
            time {string} -- [time]
        """
        self.BidList.append(bid)
        self.OfferList.append(offer)
        self.TimeList.append(time)

    def CloseDay(self):
        self.FirstPrint = self.TimeList[0]
        self.LastPrint = self.TimeList[len(self.TimeList)-1]

    def GetSimulatedPrint(self, indx):

        """Get the price print at the vector indx. Build and defines a PriceFeedPrint which 
        is returned. Checks to make sure that there is no segmentation fault
        Returns:
            [PriceFeedPrint] -- [Price feed]
            [int]            -- [Result]
        """
        if indx < len(self.BidList)-1:
            bid = float(self.BidList[indx])
            offer = float(self.OfferList[indx])
            time = str(self.TimeList[indx])
            pp = PriceFeedPrint(bid, offer, time)
            #print (pp.offer)
            return pp
        else:
            return -1

class SimulatedFeed(object):

    """Main holder for all days of data and their individual SimulatedDailyFeed objects.
    This dictionary is build in the 'buildsimulatedfeed' routine and requires the correct 
    folder and filename structire for the market in question.

    Returns:
        [type] -- [description]
    """

    def __init__(self, market = 'eurusd', numberOfDays = 3):
        self.Market = market
        self.numberOfDays = numberOfDays
        self.dataSeries = None
        #print (self.numberOfDays)
        self.FeedDays = []
        self.Feed = {}

    def SetDataSeries(self, dataSeries):
        """This set the dataSeries pointer/object from the simulation/optimisation.
        dataSeries.AddData is called in the 'Tick' routine if this field is set. Initially
        set as None
        
        Arguments:
            dataSeries {[DataSeries]} -- [DataSeries class for simualation/optimisation. See ref.]
        """
        

        self.dataSeries = dataSeries

    def BuildSimulatedFeed(self, startHour=0, endHour=24):

        """Builds the simulated feed for number of days required. Sets the time limits for the data. 
        Reads data files from MARKET_PLOT_DATA_FOLDER.
        
        Keyword Arguments:
            startHour {int} -- [description] (default: {0})
            endHour {int} -- [description] (default: {24})
        """

        print ("herererjekklk;ljerlk")
        fileList = os.listdir(MARKET_PLOT_DATA_FOLDER + "/" + self.Market)
        dayCnt = 0
        for fileName in fileList:
            print (fileName)
            if fileName.endswith(".txt"):
                fnArray = fileName.split('.')
                day = fnArray[0]
                DailyFeed = SimulatedDailyFeed(day)
                self.FeedDays.append(day)
                fullFileName = MARKET_PLOT_DATA_FOLDER + "/" + self.Market + "/" + fileName       
                try:    
                    fs = open(fullFileName, 'r')
                except Exception as e:
                    print("type error: " + str(e))
                    print(traceback.format_exc())

                for line in fs:
                    lineSpl = line.split(',')
                    
                    hr = GetHourFromPricePrint(lineSpl[0])
                    
                    #---sample data---
                    if hr> startHour:
                        if hr < endHour:
                            collectData = False
                            randNum = random.random()
                            #print (OptimsationParameters)
                            
                            if randNum < OptimsationParameters['feed_sample_rate']:
                                collectData = True


                            #-----------------
                            if collectData:
                                DailyFeed.AddPrint(lineSpl[1], lineSpl[2], lineSpl[0])
                                #print (lineSpl[1], lineSpl[2], lineSpl[0])

                fs.close()

                DailyFeed.CloseDay()
                self.Feed[day] = DailyFeed

                dayCnt +=1
                if dayCnt >= self.numberOfDays:
                    break

    def StartFeed(self):
        """Starts the feed for a particular day by setting the vector index pointer to 0.
        """
        print ("Starting feed")
        self.VIndex = 0
        return 1

 

    def Tick(self, day):
        """This simulates a market price tick, but limited to the time resolution saved in the datafiles.
        
        Arguments:
            day {[string]} -- The date of the data which is being read
            last_hr {[double/float]} -- last hour of feed
        
        Returns:
            [type] -- [description]
        """


        simulatedPricePrint = self.Feed[day].GetSimulatedPrint(self.VIndex)
        

        if self.dataSeries == None:

            if simulatedPricePrint is not -1:
                hr = GetHourFromPricePrint(simulatedPricePrint.time)
                if hr > last_hr:         
                
                    self.VIndex = 0
                    return -1
                
                print(simulatedPricePrint)
                self.VIndex += 1
                return 1

            return -1



        if self.dataSeries != None:
            
            if simulatedPricePrint is not -1:
            #add to dataseries
                #print (simulatedPricePrint, day)
                
                #hr = GetHourFromPricePrint(simulatedPricePrint.time)
                #if hr > last_hr:
                    #print ("End of day***")
                    #sleep(10000000)
                    #self.VIndex = 0
                    #self.dataSeries.DumpCandles(self.Market)
                    #print ("Reseting Market Data for new day")
                    #self.dataSeries.Reset(self.Market)

                    #return -1
                
                self.dataSeries.AddData(self.Market, simulatedPricePrint.bid, simulatedPricePrint.offer, simulatedPricePrint.time)
                
                self.VIndex += 1
                self.dataSeries.SetDay(self.Market, day)
                
                return 1
            else:
                #debug

                
                self.VIndex = 0
                #print ("Reseting Market Data for new day")
                self.dataSeries.Reset(self.Market)
                
                return -1



#debug  - build and output simulated feed

if __name__ == "__main__":
    pass
    '''
    simulatedMarket = SimulatedFeed('eurusd', 3)
    simulatedMarket.BuildSimulatedFeed(startHour=6,endHour=10)
    
    #Run without connecting to a dataseries. Simple outout to console.  
    for day in simulatedMarket.FeedDays:
        print (day)
        simulatedMarket.StartFeed()
        tickResult = 1
        while tickResult == 1:
            tickResult = simulatedMarket.Tick(day,10)
    '''




#!/usr/bin/env python

from setuptools import setup, find_packages

long_description = ""

with open("README.md", "r") as fh:
    long_description = fh.read()



setup(
    name="brahma", 
    version="0.1.413",
    author="Rahul Tandon",
    author_email="rahul@vixenintelligence.com",
    description="BrahmA Aritificial Intelligence Framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    #url="https://github.com/vixencapital/brahma/releases/tag/1.0",
    packages=['brahma', 'brahma.bots', 'brahma.dna', 'brahma.genes', 'brahma.genome', 'brahma.world'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6'
)



"""This is v1.0 of the trader template. Procedures for birth, death evolution must be contained here.
This is a template for a general intraday futures/fx trader. (obs)
"""
import os, sys, time, copy, datetime, random

#config core
#sys.path.append('../')


from brahma.configCore import *
from brahma.dna.vixen_dna import *
from brahma.genome.transcription import *

class BotRoot(object):
    
    """
    BotRoot is the root class of all indis/bots in the brahma world.
    Genetic material is deterined by lcl folder for material and derived bots  can add
    functionality. e.g.  trader  - direction of bot.

    todo: visualise bot genetic strucure

    :param object: [description]
    :type object: [type]
    :param object: [description]
    :type object: [type]
    :param object: [description]
    :type object: [type]

    """
    def __init__(self, myenv="", myspecies = "BotRoot", myargs=None):
        self.name = "VxBot" + str(random.randint(100,1000000)) 
        self.env = myenv
        self.dob = datetime.datetime.now()
        self.species = myspecies
        #genetic material
        self.parent = "Eden"
        
        self.dNA = {}           #contains strands of DNA
        #self.riskDNA = {}
        self.numberDNA = 0
        self.dNAExpression = {}         #expression value of each DNA strand 
                                        #this is required for transcription

        #self.dNARiskExpression = {}     #expression value of each risk DNA strand 
                                        #this is required for transcription

        #protein transcription
        self.transcriptionDNA = None    #this is what is used for transcription
                                        #DNA expression levels is required

        #initialised
        self.initialised = False

        #logger
        #self.logger = GetBrahmaLogger("Bot Creation")

    def __str__(self):
        """
        Overide of print method.Export high level description of bot. 

        :return: High  level description of bot.
        :rtype: String
        """
        return ("Name: {0} Number DNA: {1} Env: {2} Species: {3}".format(self.name, self.numberDNA, self.env, self.species))

    def BuildBot(self, parms = None):
        """
        Build the bot and all its genetic material. This is the root class 
        so all root genetic material is built. 

        :param parms: [description], defaults to None
        :type parms: [type], optional
        """
        print ("here")
        #logger.info("***Building Bot : {0}".format(self.name))
        
        #determine numer of dna strands
        if parms != None:
            self.numberDNA = random.randint(parms['minNumberMarketDNA'], parms['maxNumberMarketDNA'])
            
        else:
            exit(0)



        #build risk DNA - alpha - one for now.  keep simple.
        #riskDNA =  VixenDNA(deltaD = 1, dataLength=1, envName=self.env, dataDelay=1)
        #riskDNA_tag = riskDNA.name
        #riskDNA.AddRiskGenome()
        #self.riskDNA[riskDNA_tag] = riskDNA
        #self.dNARiskExpression[riskDNA_tag] = 0.0
        #del(riskDNA)
        
        
        #number of marketDNA
        for i in range(0,self.numberDNA):
            #print (self.numberDNA)
            
            #build market dna

            #get deltaT
            #dna_deltaT = random.randint(1, parms['maxDeltaT'])
            dna_deltaT = random.choice(EpochBuckets)
            
            #get length
            dna_length = random.randint(parms['minMarketDNASize'], parms['maxMarketDNASize'])
            #get delay
            dna_delay = random.randint(0, parms['maxDNADelay'])
            

            marketDNA = VixenDNA(deltaD = dna_deltaT, dataLength=dna_length, envName=self.env, dataDelay=dna_delay)
            marketDNA_tag = marketDNA.name
            #add genome... only one per DNA strand - yes for now
            marketDNA.AddGenome(parms=parms)
            # print ("dna: {}".format(i))
            # ()exit
            self.dNA[marketDNA_tag] = marketDNA
            # del(marketDNA)
            marketDNA = None
            self.dNAExpression[marketDNA_tag] = 0.0
            #print (self.MarketDNA[marketDNA_tag].ExpressionTable)
            # exit()

            

        #build transcription DNA
        self.transcriptionDNA = Transcription()
            

    def StartUp(self):
        """
        All expression levels must be set to zero before starting a new run.
        """

        for dnaTag, e in self.dNAExpression.items():
            self.dNAExpression[dnaTag] = 0.0

        #for dnaTag, e in self.dNARiskExpression.items():
        #    self.dNARiskExpression[dnaTag] = 0.0
    
    def ExpressDNA(self, data={}):
        """
        Express each strand of DNA in the bot and build the DNA epression table for
        the bot. Return values are 1 for bot is now initialised and 0 for not initialised.

        :param data: Pressure data, defaults to {} for root as no data required 
                    for test data. User defined for derived bot data requirements
        :type data: dict, optional
        :return: Initialised state of bot. True: is initialised False: not initialised 
        :rtype: Bool
        """

        #run risk first
        '''
        for dnaTag, dna in self.riskDNA.items():
            expression_value = dna.ExpressDNA(data=data)
            #check to see if all the genetic material has been initialised
            #a value of False means that one of the genomes has not been initialised
            #so ExpressDNA also returns false after settng the initialised value to  False
            if expression_value == False:
                self.initialised = False
                return  False

            self.dNARiskExpression[dnaTag] = expression_value
        '''

        #print (self.initialised)
        for dnaTag, dna in self.dNA.items():
            
            #get current data print and bucket vector--
            #bucket vector comes from derived structure (data series custom made)
            data_print = data['data_print']
            bucket_vector = data['derived_data'].GetBuckets(self.env, deltaT = dna.deltaD)

            pressure_data = {
                'current' : data_print,
                'buckets' : bucket_vector,
                'options' : data['opt']
            }

            expression_value = dna.ExpressDNA(data=pressure_data)

            #debug ---> gene flip analysis
            #print  (expression_value)
            #check to see if all the genetic material has been initialised
            #a value of False means that one of the genomes has not been initialised
            #so ExpressDNA also returns false after settng the initialised value to  False


            #update nov 2020 :  returning boolean and floating point
            if expression_value == -10:
                self.initialised = False
                return  False


            #debug ---> gene flip analysis
            #if expression_value  == 1.0:
                
            #    print  ("++++++++++++++++++++++++")
            #    exit()
            #print (expression_value)

            self.dNAExpression[dnaTag] = expression_value


        #we have expressed all DNA strands which means all genomes inside have been run
        #which means that the individual is now intialised.
        self.initialised = True

        #---debug output
        tickExpress = []
        for key, val in self.dNAExpression.items():
            #print (val)
            fp = open ("express.txt", "a")
            fp.write("{0}\n".format(val))
            fp.close()
            tickExpress.append(val)
        #print (self.initialised)
        #print (*tickExpress)
        
        #print ("***")
        #---

        return True

    def GetExpressionData(self, type=""):
        return self.dNAExpression



# class myTradeBot(BotRoot):
#   def __init__(self, myenv="", myspecies = "myTradeBot", myargs=None):
#     super().__init__(myenv=myenv, myspecies = myspecies, myargs=myargs)
    
#     #looking for a buyer here
#     self.direction = 1

#   #def ExpressDNA(self):
#   #  pass




"""
Module holding root data feed for the brahma world environment.
Brahma allows for both static and data feeds to the virtual world 
with both 'sets' of data provided by a data feed. We define the root data
feed class here.

The data heirarchy in the Brahma world is divided into 4 main categories, each which must
be customised by the user for their bots/indis.

1 ->  Data Source. e.g. market prices. static or live.
!!!!!!!!!!!!!!!!!!!!!!!!
2 ->  NO!!!!Buckets: Data Source is then bundled into groups or "buckets" of data!!! This
      comes after and is handles by a bucket class for DNA to get correct data vector...
!!!!!!!!!!!!!!!!!!!!!!!!
3 ->  Epochs:  Data sources in static feeds are allowed to be divided into epochs of data.
      e.g. days of market data. At least one epoch of data is required.
3 ->  Data prints for feeding into the world. This is the stage at which data is presented into the world 
      for the bots/indis to interact with. Data must be declared and presented as a "print" which
      is read by an indi/bot for genetic evaluation.

Ignore "3" for live data feeds into the Brahma World.
"""

#------------------DERIVATION REQUIRED-----------
import time, sys

# #config core
# sys.path.append('../')
# from configCore import *

class BrahmaData(object):
  def __init__(self, type='static'):
    self.epochs = None     #list of epochs. Each  epoch has its own feed to the world

  def AddEpochs(self, epochs  = None):
    self.epochs = epochs

class DataPrint(object):
  """
  Root data print  class
  :param object: [description]
  :type object: [type]
  """

  def __init__(self, value=0, high=0, low=0):
    self.value =  value
    self.high = 0
    self.low  = 0

  def __exit__(self):
    pass

class BrahmaLiveData(object):
  def __init__(self, data_type = 'dynamic', source_file = "live.data"):
    self.dataType = data_type
    self.epoch = None
    self.sourceDataFile = source_file
    self.dataSource = LiveDataSource(source_file=source_file)
    self.running = False


  def Start(self):
    self.epoch = BrahmaEpoch(self.dataSource.dataSource)
    self.running = True

  def Tick(self):
    
    #get new values from file 
    new_data = self.dataSource.Tick()
    
    #create print which is read by bots/indis
    dp = testPricePrint(new_data[0])
    #update epoch and price feed to iter over
    self.epoch.Update(data=dp)

  def Stop(self):
    self.running = False

class LiveDataSource(object):
  def __init__(self, data = [], source_file = ""):
    self.dataSource =  data #list of data prints
    self.data_source_file = source_file

  def Tick(self):
    #update data source - grab new data and updata data. This is done from file here.
    data = self.GrabData()
    
    return data

  def GrabData(self):
    return_data = []
    with open (self.data_source_file, 'r') as f:
      lines = f.read().splitlines()
      last_line = lines[-1]
      #print (last_line)
      data = last_line.split(',')
      #print (data[0])
      return_data.append(data)
    
    #print("d {0}".format(return_data))
    
      
    return return_data
    
#-------------------NO DERIVATION-----------------


class DataSource(object):
  def __init__(self, data = []):
    self.dataSource =  data #list of data prints

class BrahmaEpoch(object):

  def __init__(self, data = None, epochTag = "root", start_idx = 0, end_idx = 0):
    self.source = data
    self.epochTag = epochTag
    self.feed = BrahmaDataFeed(self.source, self.epochTag, start_index = start_idx, end_index= end_idx)

  def Update(self, data = None):
    """
    For live data - update the feed object.

    :param data: [description], defaults to []
    :type data: list, optional
    """
    #print (data)
    self.source.append(data)
    
    

  def __str__(self):
    return ("Ind epoch Tag: {0} ".format(self.epochTag))

class Epochs(object):
  def __init__(self):
    self.epochs = []
    self.startIndex = 0
    self.index = self.startIndex

  def AddEpoch(self, newEpoch = None):
    self.epochs.append(newEpoch)

  def __iter__(self):
    return self

  def __next__(self):
    
    
    if self.index >= len(self.epochs):
      self.index = self.startIndex
      raise StopIteration
    returnFeed = self.epochs[self.index].feed
    self.index+=1
    return returnFeed


  def __str__(self):
    return ("Number of Epochs: {0}".format(len(self.epochs)))

class BrahmaDataFeed(object):
  """
  Class responsible for connecting the Brahma World to the 'environmental' data source
  for the virtual world. Environmental pressure is provided via this data and in order to allow
  fully customizable data sets and environments, this class provides the conduit of any data
  source to its corresponding. This data feed class is responsible for introducing data prints 
  into the brahma world.

  :param object: root class
  :type object: object
  """

  def __init__(self, source = None, epoch_tag = "unique", start_index = 0, end_index = 0):
    
    self.start_index = start_index
    self.end_index = end_index
    self.source = source
    self.epoch_tag = epoch_tag
    self.index  = self.start_index


  def __exit__(self):
    pass

  def __iter__(self):
    return self
    
  def __next__(self):
    #print (self.source)
    if self.index >= len(self.source):
      #print ("No values available. Data exhausted.")
      self.index = self.start_index
      raise StopIteration
    print_value = self.source[self.index]
    self.index += 1
    return print_value
  '''
  def Tick(self, data = None):
    if data != None:
      print (data)
      print (self.source)
      print (len(self.source))
      self.source.append(data)
      print (self.source)
      print (len(self.source))
      exit()
  '''

  def __str__(self):
    return ("source len: {0} epoch tag: {1}".format(len(self.source), self.epoch_tag))

  """
  Root data  source for Brahma World.

  :param object: [description]
  :type object: [type]
  """

class testPricePrint(object):
  def __init__(self, data):
   
    
    self.one =data[0]
    self.two = data[1]
    self.three = data[2]

  def __str__(self):
    return "pp: {0} {1} {2}".format(self.one, self.two, self.three)



if __name__ == "__main__":

  '''  static data example'''
  # source1 = [0,3,5,2,5,6]
  # source2 = [4,23,35,52,35,36]
  # source3 = [50,33,35,52,55,26]
  # source4 = [0,3,5,2,5,6]

  # epoch_1 = BrahmaEpoch(source1, "1")
  # epoch_2 = BrahmaEpoch(source2, "2")
  # epoch_3 = BrahmaEpoch(source3, "3")
  # epoch_4 = BrahmaEpoch(source4, "4")


  # my_epochs = Epochs()
  # my_epochs.AddEpoch(epoch_1)
  # my_epochs.AddEpoch(epoch_2)
  # my_epochs.AddEpoch(epoch_3)
  # my_epochs.AddEpoch(epoch_4)


  '''static data example'''
  #1.  define data print you will be using
  #we will  use the root one here

  #2. Build data prints - this is the main  data source
  p1 = DataPrint(15,73,2)
  p2 = DataPrint(37,36,2)
  p3 = DataPrint(14,33,42)
  p4 = DataPrint(12,53,23)
  p5 = DataPrint(13,34,32)
  p6 = DataPrint(13,33,2)
  p7 = DataPrint(1,33,2)
  p8 = DataPrint(15,33,2)
  p9 = DataPrint(16,43,2)
  p10 = DataPrint(13,3,2)
  p11 = DataPrint(12,23,2)
  p12 = DataPrint(11,34,32)


  #3. Add data to data source
  dataSource1 = DataSource(data=[p1,p3,p2])
  # print(len(dataSource1.dataSource))
  dataSource2 = DataSource(data=[p4,p5,p6])
  dataSource3 = DataSource(data=[p7,p8,p9])

  #4. place into indvidual epochs
  e1 = BrahmaEpoch(data=dataSource1.dataSource,epochTag="1")

  e2 = BrahmaEpoch(data=dataSource2.dataSource,epochTag="2")
  e3 = BrahmaEpoch(data=dataSource3.dataSource,epochTag="3")

  #5. group epochs together for the world data
  world_data_epochs = Epochs()
  world_data_epochs.AddEpoch(e1)
  world_data_epochs.AddEpoch(e2)
  world_data_epochs.AddEpoch(e3)

  #6. create world data structure which will feed the virtual world  via epochs and data feeds
  world_data = BrahmaData(type='static')
  world_data.AddEpochs(world_data_epochs)

  #7.  feed the world
  # print (world_data.epochs.epochs[0].feed)
  print (world_data.epochs)
  for epoch in world_data.epochs:
    for dp in  epoch:
      pass

  '''
  #---LIVE DATA FEED---
  print ("live feed test")

  #1. build live data
  liveData = BrahmaLiveData(source_file='live.data')

  #2. start the feed
  liveData.Start()

  # tick through data
  #liveData.Tick()

  liveData.Tick()
  
  for dp in liveData.epoch.feed:
    liveData.Tick()
    time.sleep(1)
    print (dp)

'''





  



  
  




  

